@@ -1,4 +1,4 @@
-/* $Id: tftp.c,v 1.12 2001/11/13 19:58:32 hpa Exp $ */
+/* $Id: tftp.c,v 1.14 2001/11/29 22:02:52 hpa Exp $ */
 
 /* $OpenBSD: tftp.c,v 1.4 1997/08/06 06:43:45 deraadt Exp $	*/
 /* $NetBSD: tftp.c,v 1.5 1995/04/29 05:55:25 cgd Exp $	*/
@@ -42,7 +42,7 @@
 /* static char sccsid[] = "@(#)tftp.c	8.1 (Berkeley) 6/6/93"; */
 /* static char rcsid[] = "$OpenBSD: tftp.c,v 1.4 1997/08/06 06:43:45 deraadt Exp $"; */
 static const char *rcsid UNUSED =
-"tftp-hpa $Id: tftp.c,v 1.12 2001/11/13 19:58:32 hpa Exp $";
+"tftp-hpa $Id: tftp.c,v 1.14 2001/11/29 22:02:52 hpa Exp $";
 #endif /* not lint */
 
 /* Many bug fixes are from Jim Guyton <guyton@rand-unix> */
@@ -50,21 +50,6 @@ static const char *rcsid UNUSED =
 /*
  * TFTP User Program -- Protocol Machines
  */
-#include <sys/types.h>
-#include <sys/socket.h>
-#include <sys/time.h>
-
-#include <netinet/in.h>
-
-#include <arpa/tftp.h>
-
-#include <errno.h>
-#include <setjmp.h>
-#include <signal.h>
-#include <stdio.h>
-#include <string.h>
-#include <unistd.h>
-
 #include "extern.h"
 
 extern  struct sockaddr_in peeraddr;	/* filled in by main */
@@ -92,7 +77,7 @@ static void tpacket(const char *, struct tftphdr *, int);
  * Send the requested file.
  */
 void
-tftp_sendfile(int fd, char *name, char *mode)
+tftp_sendfile(int fd, const char *name, const char *mode)
 {
 	struct tftphdr *ap;	   /* data and ack packets */
 	struct tftphdr *dp;
@@ -108,8 +93,8 @@ tftp_sendfile(int fd, char *name, char *mode)
 	startclock();		/* start stat's clock */
 	dp = r_init();		/* reset fillbuf/read-ahead code */
 	ap = (struct tftphdr *)ackbuf;
-	file = fdopen(fd, "r");
 	convert = !strcmp(mode, "netascii");
+	file = fdopen(fd, convert ? "rt" : "rb");
 	block = 0;
 	is_request = 1;		/* First packet is the actual WRQ */
 	amount = 0;
@@ -200,7 +185,7 @@ abort:
  * Receive a file.
  */
 void
-tftp_recvfile(int fd, char *name, char *mode)
+tftp_recvfile(int fd, const char *name, const char *mode)
 {
 	struct tftphdr *ap;
 	struct tftphdr *dp;
@@ -216,8 +201,8 @@ tftp_recvfile(int fd, char *name, char *mode)
 	startclock();
 	dp = w_init();
 	ap = (struct tftphdr *)ackbuf;
-	file = fdopen(fd, "w");
 	convert = !strcmp(mode, "netascii");
+	file = fdopen(fd, convert ?"wt":"wb");
 	block = 1;
 	firsttrip = 1;
 	amount = 0;
