@@ -1,4 +1,4 @@
-/* tftp-hpa: $Id: tftpd.c,v 1.42 2001/11/20 19:45:12 hpa Exp $ */
+/* tftp-hpa: $Id: tftpd.c,v 1.46 2001/11/29 22:07:50 hpa Exp $ */
 
 /* $OpenBSD: tftpd.c,v 1.13 1999/06/23 17:01:36 deraadt Exp $	*/
 
@@ -45,7 +45,7 @@ static const char *copyright UNUSED =
 /*static char sccsid[] = "from: @(#)tftpd.c	5.13 (Berkeley) 2/26/91";*/
 /*static char rcsid[] = "$OpenBSD: tftpd.c,v 1.13 1999/06/23 17:01:36 deraadt Exp $: tftpd.c,v 1.6 1997/02/16 23:49:21 deraadt Exp $";*/
 static const char *rcsid UNUSED =
-"tftp-hpa $Id: tftpd.c,v 1.42 2001/11/20 19:45:12 hpa Exp $";
+"tftp-hpa $Id: tftpd.c,v 1.46 2001/11/29 22:07:50 hpa Exp $";
 #endif /* not lint */
 
 /*
@@ -56,7 +56,6 @@ static const char *rcsid UNUSED =
 
 #include <sys/ioctl.h>
 #include <signal.h>
-#include <fcntl.h>
 #include <netdb.h>
 #include <ctype.h>
 #include <pwd.h>
@@ -171,6 +170,25 @@ read_remap_rules(const char *file)
 }
 #endif
 
+static inline void
+set_socket_nonblock(int fd, int flag)
+{
+  int err;
+  int flags;
+#if defined(HAVE_FCNTL) && defined(HAVE_O_NONBLOCK_DEFINITION)
+  /* Posixly correct */
+  err = ((flags = fcntl(fd, F_GETFL, 0)) < 0) ||
+    (fcntl(fd, F_SETFL, flag ? flags|O_NONBLOCK : flags&~O_NONBLOCK) < 0);
+#else
+  flags = flag ? 1 : 0;
+  err = (ioctl(fd, FIONBIO, &flags) < 0);
+#endif
+  if ( err ) {
+    syslog(LOG_ERR, "Cannot set nonblock flag on socket: %m");
+    exit(EX_OSERR);
+  }
+}
+
 int
 main(int argc, char **argv)
 {
@@ -180,7 +198,6 @@ main(int argc, char **argv)
   struct sockaddr_in myaddr;
   struct sockaddr_in bindaddr;
   int n;
-  int on = 1;
   int fd = 0;
   int standalone = 0;		/* Standalone (listen) mode */
   char *address = NULL;		/* Address to listen to */
@@ -298,10 +315,11 @@ main(int argc, char **argv)
   if ( spec_umask || !unixperms )
     umask(my_umask);
   
-  if (ioctl(fd, FIONBIO, &on) < 0) {
-    syslog(LOG_ERR, "ioctl(FIONBIO): %m");
-    exit(EX_OSERR);
-  }
+  /* Note: on Cygwin, select() on a nonblocking socket becomes
+     a nonblocking select. */
+#ifndef __CYGWIN__
+  set_socket_nonblock(fd, 1);
+#endif
 
 #ifdef WITH_REGEX
   if ( rewrite_file )
@@ -362,15 +380,32 @@ main(int argc, char **argv)
     /* Daemonize this process */
     {
       pid_t f = fork();
+      int nfd;
       if ( f > 0 )
 	exit(0);
       if ( f < 0 ) {
 	syslog(LOG_ERR, "cannot fork: %m");
 	exit(EX_OSERR);
       }
-      close(0); close(1); close(2);
+      nfd = open("/dev/null", O_RDWR);
+      if ( nfd >= 0 ) {
+#ifdef HAVE_DUP2
+	dup2(nfd, 0);
+	dup2(nfd, 1);
+	dup2(nfd, 2);
+#else
+	close(0); dup(nfd);
+	close(1); dup(nfd);
+	close(2); dup(nfd);
+#endif
+	close(nfd);
+      } else {
+	close(0); close(1); close(2);
+      }
 #ifdef HAVE_SETSID
+#ifndef __CYGWIN__		/* Kills the process on Cygwin? */
       setsid();
+#endif
 #endif
     }
   } else {
@@ -415,33 +450,47 @@ main(int argc, char **argv)
     tv_waittime.tv_sec = waittime;
     tv_waittime.tv_usec = 0;
     
+#ifdef __CYGWIN__
+    /* On Cygwin, select() on a nonblocking socket returns immediately,
+       with a rv of 0! */
+    set_socket_nonblock(fd, 0);
+#endif    
+
     /* Never time out if we're in standalone mode */
     rv = select(fd+1, &readset, NULL, NULL, standalone ? NULL : &tv_waittime);
     if ( rv == -1 && errno == EINTR )
       continue;		/* Signal caught, reloop */
     if ( rv == -1 ) {
       syslog(LOG_ERR, "select loop: %m");
-      exit(EX_OSERR);
+      exit(EX_IOERR);
     } else if ( rv == 0 ) {
       exit(0);		/* Timeout, return to inetd */
     }
+
+#ifdef __CYGWIN__
+    set_socket_nonblock(fd, 1);
+#endif    
     
     fromlen = sizeof (from);
     n = myrecvfrom(fd, buf, sizeof (buf), 0,
 		   (struct sockaddr *)&from, &fromlen,
 		   &myaddr);
 
+    if ( n < 0 ) {
+      if ( E_WOULD_BLOCK(errno) || errno == EINTR ) {
+	continue;		/* Again, from the top */
+      } else {
+	syslog(LOG_ERR, "recvfrom: %m");
+	exit(EX_IOERR);
+      }
+    }
+
     if ( standalone && myaddr.sin_addr.s_addr == INADDR_ANY ) {
       /* myrecvfrom() didn't capture the source address; but we might
 	 have bound to a specific address, if so we should use it */
       memcpy(&myaddr.sin_addr, &bindaddr.sin_addr, sizeof bindaddr.sin_addr);
     }
 
-    if (n < 0) {
-      syslog(LOG_ERR, "recvfrom: %m");
-      exit(EX_IOERR);
-    }
-    
     /*
      * Now that we have read the request packet from the UDP
      * socket, we fork and go back to listening to the socket.
@@ -509,10 +558,14 @@ main(int argc, char **argv)
 #endif
 
   /* Chroot and drop privileges */
-  
-  if (secure && chroot(".")) {
-    syslog(LOG_ERR, "chroot: %m");
-    exit(EX_OSERR);
+  if (secure) {
+    if (chroot(".")) {
+      syslog(LOG_ERR, "chroot: %m");
+      exit(EX_OSERR);
+    }
+#ifdef __CYGWIN__
+    chdir("/");			/* Cygwin chroot() bug workaround */
+#endif
   }
 
 #ifdef HAVE_SETREGID
@@ -876,7 +929,7 @@ FILE *file;
  * Validate file access.  Since we
  * have no uid or gid, for now require
  * file to exist and be publicly
- * readable/writable.
+ * readable/writable, unless -p specified.
  * If we were invoked with arguments
  * from inetd then the file must also be
  * in one of the given directory prefixes.
@@ -888,9 +941,10 @@ validate_access(char *filename, int mode, struct formats *pf)
 {
   struct stat stbuf;
   int   i, len;
-  int	fd, wmode;
+  int	fd, wmode, rmode;
   char *cp;
   const char **dirp;
+  char stdio_mode[3];
   
   tsize_ok = 0;
   
@@ -921,9 +975,12 @@ validate_access(char *filename, int mode, struct formats *pf)
    */
   wmode = O_WRONLY |
     (cancreate ? O_CREAT : 0) |
-    (unixperms ? O_TRUNC : 0);
+    (unixperms ? O_TRUNC : 0) |
+    (pf->f_convert ? O_TEXT : O_BINARY);
+  rmode = O_RDONLY |
+    (pf->f_convert ? O_TEXT : O_BINARY);
 
-  fd = open(filename, mode == RRQ ? O_RDONLY : wmode, 0666);
+  fd = open(filename, mode == RRQ ? rmode : wmode, 0666);
   if (fd < 0) {
     switch (errno) {
     case ENOENT:
@@ -962,7 +1019,11 @@ validate_access(char *filename, int mode, struct formats *pf)
     tsize_ok = 1;
   }
 
-  file = fdopen(fd, (mode == RRQ)? "r":"w");
+  stdio_mode[0] = (mode == RRQ) ? 'r':'w';
+  stdio_mode[1] = (pf->f_convert) ? 't':'b';
+  stdio_mode[2] = '\0';
+
+  file = fdopen(fd, stdio_mode);
   if (file == NULL)
     exit(EX_OSERR);		/* Internal error */
 
